==> generic-plugin-3.6.0-sources.jar: com/xebialabs/deployit/plugin/generic/deployed/CopiedArtifact.java <==
package com.xebialabs.deployit.plugin.generic.deployed;

import com.xebialabs.deployit.plugin.api.deployment.planning.Create;
import com.xebialabs.deployit.plugin.api.deployment.planning.DeploymentPlanningContext;
import com.xebialabs.deployit.plugin.api.deployment.planning.Destroy;
import com.xebialabs.deployit.plugin.api.deployment.planning.Modify;
import com.xebialabs.deployit.plugin.api.deployment.specification.Delta;
import com.xebialabs.deployit.plugin.api.udm.DeployableArtifact;
import com.xebialabs.deployit.plugin.api.udm.Metadata;
import com.xebialabs.deployit.plugin.api.udm.Property;
import com.xebialabs.deployit.plugin.api.udm.artifact.Artifacts;
import com.xebialabs.deployit.plugin.api.udm.artifact.DerivedArtifact;
import com.xebialabs.deployit.plugin.api.udm.artifact.PlaceholderReplacer;
import com.xebialabs.deployit.plugin.api.validation.Placeholders;
import com.xebialabs.deployit.plugin.generic.step.ArtifactCopyStep;
import com.xebialabs.deployit.plugin.generic.step.ArtifactDeleteStep;
import com.xebialabs.overthere.OverthereFile;

import java.util.Map;

import static com.google.common.collect.Maps.newHashMap;

@SuppressWarnings("serial")
@Metadata(virtual = true, description = "An artifact deployed on a generic container")
@Placeholders
public class CopiedArtifact<D extends DeployableArtifact> extends AbstractDeployedArtifact<D> implements DerivedArtifact<D> {

    private boolean useDescriptionGeneratedByStep = false;

    private OverthereFile placeholderProcessedFile;

    @Property(description = "A Map containing all the placeholders mapped to their values. Special values are &lt;ignore&gt; or &lt;empty&gt;", required = false)
    private Map<String, String> placeholders = newHashMap();

    @Override
    public Map<String, String> getPlaceholders() {
        return placeholders;
    }

    @Override
    public void setPlaceholders(Map<String, String> placeholders) {
        this.placeholders = placeholders;
    }

    @Override
    public OverthereFile getFile() {
        return placeholderProcessedFile;
    }

    @Override
    public void setFile(OverthereFile file) {
        this.placeholderProcessedFile = file;
    }

    @Override
    public D getSourceArtifact() {
        return getDeployable();
    }

    @Override
    public void initFile(PlaceholderReplacer replacer) {
        Artifacts.replacePlaceholders(this, replacer);
    }

    @Create
    public void executeCreate(DeploymentPlanningContext ctx) {
        ctx.addStep(createArtifactCopyStep());
    }

    protected ArtifactCopyStep createArtifactCopyStep() {
        ArtifactCopyStep step = new ArtifactCopyStep(getCreateOrder(), getFile(), getContainer(), getTargetDirectory());
        step.setCreateTargetPath(isCreateTargetDirectory());
        step.setTargetFileName(resolveTargetFileName());
        step.setSourceFileDescription(getDeployable().getName());
        if (!useDescriptionGeneratedByStep)
            step.setDescription(getDescription(getCreateVerb()));
        return step;
    }

    @Modify
    public void executeModify(DeploymentPlanningContext ctx, Delta d) {
        @SuppressWarnings("unchecked")
        CopiedArtifact<D> old = (CopiedArtifact<D>) d.getPrevious();
        old.executeDestroy(ctx);
        executeCreate(ctx);
    }

    @Destroy
    public void executeDestroy(DeploymentPlanningContext ctx) {
        ctx.addStep(createArtifactDeleteStep());
    }

    protected ArtifactDeleteStep createArtifactDeleteStep() {
        ArtifactDeleteStep step = new ArtifactDeleteStep(getDestroyOrder(), getContainer(), getDeployable(), getTargetDirectory());
        step.setTargetDirectoryShared(isTargetDirectoryShared());
        step.setTargetFile(resolveTargetFileName());
        if (!useDescriptionGeneratedByStep)
            step.setDescription(getDescription(getDestroyVerb()));
        return step;
    }

    public void setUseDescriptionGeneratedByStep(boolean useDescriptionGeneratedByStep) {
        this.useDescriptionGeneratedByStep = useDescriptionGeneratedByStep;
    }

}

==> generic-plugin-3.8.1-sources.jar: com/xebialabs/deployit/plugin/generic/deployed/CopiedArtifact.java <==
package com.xebialabs.deployit.plugin.generic.deployed;

import java.util.Map;

import com.xebialabs.deployit.plugin.api.deployment.planning.Create;
import com.xebialabs.deployit.plugin.api.deployment.planning.DeploymentPlanningContext;
import com.xebialabs.deployit.plugin.api.deployment.planning.Destroy;
import com.xebialabs.deployit.plugin.api.deployment.planning.Modify;
import com.xebialabs.deployit.plugin.api.deployment.specification.Delta;
import com.xebialabs.deployit.plugin.api.deployment.specification.Operation;
import com.xebialabs.deployit.plugin.api.flow.Step;
import com.xebialabs.deployit.plugin.api.udm.DeployableArtifact;
import com.xebialabs.deployit.plugin.api.udm.Metadata;
import com.xebialabs.deployit.plugin.api.udm.Property;
import com.xebialabs.deployit.plugin.api.udm.artifact.DerivedArtifact;
import com.xebialabs.deployit.plugin.api.validation.Placeholders;
import com.xebialabs.deployit.plugin.generic.step.ArtifactCopyStep;
import com.xebialabs.deployit.plugin.generic.step.ArtifactDeleteStep;
import com.xebialabs.deployit.plugin.generic.step.SuccessStep;
import com.xebialabs.overthere.OverthereFile;

import static com.google.common.collect.Maps.newHashMap;
import static java.lang.String.format;

@SuppressWarnings("serial")
@Metadata(virtual = true, description = "An artifact deployed on a generic container")
@Placeholders
public class CopiedArtifact<D extends DeployableArtifact> extends AbstractDeployedArtifact<D> implements DerivedArtifact<D> {

    private boolean useDescriptionGeneratedByStep = false;

    private OverthereFile placeholderProcessedFile;

    @Property(defaultValue = "false", required = false, hidden = true, description = "if true, preserve the existing files on the remote host (do not delete the copied files during the destroy operation). ")
    private boolean preserveExistingFiles;

    @Property(description = "A Map containing all the placeholders mapped to their values. Special values are &lt;ignore&gt; or &lt;empty&gt;", required = false)
    private Map<String, String> placeholders = newHashMap();

    @Override
    public Map<String, String> getPlaceholders() {
        return placeholders;
    }

    @Override
    public void setPlaceholders(Map<String, String> placeholders) {
        this.placeholders = placeholders;
    }

    @Override
    public OverthereFile getFile() {
        return placeholderProcessedFile;
    }

    @Override
    public void setFile(OverthereFile file) {
        this.placeholderProcessedFile = file;
    }

    @Override
    public D getSourceArtifact() {
        return getDeployable();
    }

    @Create
    public void executeCreate(DeploymentPlanningContext ctx, Delta delta) {
        Step step = createArtifactCopyStep();
        ctx.addStepWithCheckpoint(step, delta, Operation.CREATE);
    }

    protected ArtifactCopyStep createArtifactCopyStep() {
        ArtifactCopyStep step = new ArtifactCopyStep(getCreateOrder(), getFile(), getContainer(), getTargetDirectory());
        step.setCreateTargetPath(isCreateTargetDirectory());
        step.setTargetFileName(resolveTargetFileName());
        step.setSourceFileDescription(getDeployable().getName());
        step.setPreserveExistingFiles(preserveExistingFiles);
        if (!useDescriptionGeneratedByStep) {
            step.setDescription(getDescription(getCreateVerb()));
        }
        return step;
    }

    @Modify
    public void executeModify(DeploymentPlanningContext ctx, Delta delta) {
        @SuppressWarnings("unchecked")
        CopiedArtifact<D> old = (CopiedArtifact<D>) delta.getPrevious();
        old.executeDestroy(ctx, delta);
        executeCreate(ctx, delta);
    }

    @Destroy
    public void executeDestroy(DeploymentPlanningContext ctx, Delta delta) {
        final Step step;
        if (!preserveExistingFiles) {
            step = createArtifactDeleteStep();
        } else {
            step = new SuccessStep(getDestroyOrder(), format("Preserve the existing files %s on %s", getTargetDirectory(), getContainer()));
        }
        ctx.addStepWithCheckpoint(step, delta, Operation.DESTROY);
    }

    protected ArtifactDeleteStep createArtifactDeleteStep() {
        ArtifactDeleteStep step = new ArtifactDeleteStep(getDestroyOrder(), getContainer(), getDeployable(), getTargetDirectory());
        step.setTargetDirectoryShared(isTargetDirectoryShared());
        step.setTargetFile(resolveTargetFileName());
        if (!useDescriptionGeneratedByStep) {
            step.setDescription(getDescription(getDestroyVerb()));
        }
        return step;
    }

    public void setUseDescriptionGeneratedByStep(boolean useDescriptionGeneratedByStep) {
        this.useDescriptionGeneratedByStep = useDescriptionGeneratedByStep;
    }

}
